import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;

public class CayleyMenger {

    /**These are the  main functions.  The arguments
       are normalized so that the first coordinate is 1.
       These correspond to the functions cm and dcm above*/


    /**The Cayley-Menger determinant*/

    public static double f(double[] d) {
	double d12=d[0];
	double d13=d[1];
	double d14=d[2];
	double d23=d[3];
	double d24=d[4];
	double d34=d[5];

double y=-2*p(d12,2)*p(d13,2)*p(d23,2) + 
   2*p(d12,2)*p(d14,2)*p(d23,2) + 
   2*p(d13,2)*p(d14,2)*p(d23,2) - 2*p(d14,4)*p(d23,2) - 
   2*p(d14,2)*p(d23,4) + 2*p(d12,2)*p(d13,2)*p(d24,2) - 
   2*p(d13,4)*p(d24,2) - 2*p(d12,2)*p(d14,2)*p(d24,2) + 
   2*p(d13,2)*p(d14,2)*p(d24,2) + 
   2*p(d13,2)*p(d23,2)*p(d24,2) + 
   2*p(d14,2)*p(d23,2)*p(d24,2) - 2*p(d13,2)*p(d24,4) - 
   2*p(d12,4)*p(d34,2) + 2*p(d12,2)*p(d13,2)*p(d34,2) + 
   2*p(d12,2)*p(d14,2)*p(d34,2) - 
   2*p(d13,2)*p(d14,2)*p(d34,2) + 
   2*p(d12,2)*p(d23,2)*p(d34,2) + 
   2*p(d14,2)*p(d23,2)*p(d34,2) + 
   2*p(d12,2)*p(d24,2)*p(d34,2) + 
   2*p(d13,2)*p(d24,2)*p(d34,2) - 
   2*p(d23,2)*p(d24,2)*p(d34,2) - 2*p(d12,2)*p(d34,4);
   return(y);
    }


    public static double df(int[] q,double[] d) {
	double s=0;
	for(int i=0;i<6;++i) {
	    s=s+q[i]*df(i,d);
	}
	return(s);
    }


    /**the directional derivatives*/

    public static double df(int k,double[] d) {
	if(k==0) return(d0f(d));
	if(k==1) return(d1f(d));
	if(k==2) return(d2f(d));
	if(k==3) return(d3f(d));
	if(k==4) return(d4f(d));
	if(k==5) return(d5f(d));
	return(0);
    }



    public static double d0f(double[] d) {
	double d12=d[0];
	double d13=d[1];
	double d14=d[2];
	double d23=d[3];
	double d24=d[4];
	double d34=d[5];

double y=-4*d12*p(d13,2)*p(d23,2) + 4*d12*p(d14,2)*p(d23,2) + 
   4*d12*p(d13,2)*p(d24,2) - 4*d12*p(d14,2)*p(d24,2) - 
   8*p(d12,3)*p(d34,2) + 4*d12*p(d13,2)*p(d34,2) + 
   4*d12*p(d14,2)*p(d34,2) + 4*d12*p(d23,2)*p(d34,2) + 
	4*d12*p(d24,2)*p(d34,2) - 4*d12*p(d34,4);
    return(y);
}


    public static double d1f(double[] d) {
	double d12=d[0];
	double d13=d[1];
	double d14=d[2];
	double d23=d[3];
	double d24=d[4];
	double d34=d[5];

double y=-4*p(d12,2)*d13*p(d23,2) + 4*d13*p(d14,2)*p(d23,2) + 
   4*p(d12,2)*d13*p(d24,2) - 8*p(d13,3)*p(d24,2) + 
   4*d13*p(d14,2)*p(d24,2) + 4*d13*p(d23,2)*p(d24,2) - 
   4*d13*p(d24,4) + 4*p(d12,2)*d13*p(d34,2) - 
    4*d13*p(d14,2)*p(d34,2) + 4*d13*p(d24,2)*p(d34,2);
    return(y);
}


    public static double d2f(double[] d) {
	double d12=d[0];
	double d13=d[1];
	double d14=d[2];
	double d23=d[3];
	double d24=d[4];
	double d34=d[5];

double y=4*p(d12,2)*d14*p(d23,2) + 4*p(d13,2)*d14*p(d23,2) - 
   8*p(d14,3)*p(d23,2) - 4*d14*p(d23,4) - 
   4*p(d12,2)*d14*p(d24,2) + 4*p(d13,2)*d14*p(d24,2) + 
   4*d14*p(d23,2)*p(d24,2) + 4*p(d12,2)*d14*p(d34,2) - 
    4*p(d13,2)*d14*p(d34,2) + 4*d14*p(d23,2)*p(d34,2);
    return(y);
}


    public static double d3f(double[] d) {
	double d12=d[0];
	double d13=d[1];
	double d14=d[2];
	double d23=d[3];
	double d24=d[4];
	double d34=d[5];

double y=-4*p(d12,2)*p(d13,2)*d23 + 4*p(d12,2)*p(d14,2)*d23 + 
   4*p(d13,2)*p(d14,2)*d23 - 4*p(d14,4)*d23 - 
   8*p(d14,2)*p(d23,3) + 4*p(d13,2)*d23*p(d24,2) + 
   4*p(d14,2)*d23*p(d24,2) + 4*p(d12,2)*d23*p(d34,2) + 
    4*p(d14,2)*d23*p(d34,2) - 4*d23*p(d24,2)*p(d34,2);
    return(y);
}

    public static double d4f(double[] d) {
	double d12=d[0];
	double d13=d[1];
	double d14=d[2];
	double d23=d[3];
	double d24=d[4];
	double d34=d[5];

double y=4*p(d12,2)*p(d13,2)*d24 - 4*p(d13,4)*d24 - 
   4*p(d12,2)*p(d14,2)*d24 + 4*p(d13,2)*p(d14,2)*d24 + 
   4*p(d13,2)*p(d23,2)*d24 + 4*p(d14,2)*p(d23,2)*d24 - 
   8*p(d13,2)*p(d24,3) + 4*p(d12,2)*d24*p(d34,2) + 
    4*p(d13,2)*d24*p(d34,2) - 4*p(d23,2)*d24*p(d34,2);
    return(y);
}


    public static double d5f(double[] d) {
	double d12=d[0];
	double d13=d[1];
	double d14=d[2];
	double d23=d[3];
	double d24=d[4];
	double d34=d[5];

double y=-4*p(d12,4)*d34 + 4*p(d12,2)*p(d13,2)*d34 + 
   4*p(d12,2)*p(d14,2)*d34 - 4*p(d13,2)*p(d14,2)*d34 + 
   4*p(d12,2)*p(d23,2)*d34 + 4*p(d14,2)*p(d23,2)*d34 + 
   4*p(d12,2)*p(d24,2)*d34 + 4*p(d13,2)*p(d24,2)*d34 - 
    4*p(d23,2)*p(d24,2)*d34 - 8*p(d12,2)*p(d34,3);
    return(y);
}


    public static double p(double x,int k) {
	double y=Math.pow(x,k);
	return(y);
    }


    /**This routine computes the Cayley Menger determinant of a
       4x4 symmetric matrix with 0's along the diagonal.  One
       squares the matrix entries, puts it into a 5x5 matrix
       and then takes the determinant*/

    public static Matrix cm(double[][] d) {
       Matrix m=new Matrix();
       m.a[0][0]=0;
       for(int i=1;i<5;++i) {
           m.a[0][i]=1;
	   m.a[i][0]=1;
       }

       for(int i=1;i<5;++i) {
	  for(int j=1;j<5;++j) { 
	      double t=d[i-1][j-1]; 
	      m.a[i][j]=t*t;
	  }
       }
       return(m);
    }



}




