import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


/**This is essentially a cleaner version of the
   first test in the PositiveTester class*/


public class NegativeFinder implements Runnable {
    PictureCanvas P;
    int GROUP,TOTAL;
    int[][] LIST=new int[48][4];
    boolean HALT;
    Poly6 CM;
    Poly6 DCM;
    int[] DIFF;

    public NegativeFinder() {
	HALT=true;
    }

    public NegativeFinder(PictureCanvas PP) {
	HALT=true;
	this.P=PP;
    }

    /**Main routine*/
    public void run() {
	HALT=false;
	preProcess();
	int count=0;
	while(HALT==false) {
	    ++count;
            process();
	    if(count%10000==0) System.out.println("count "+count);
	}
    }

    public void preProcess() {
	GROUP=P.GROUP.mode;
	if(GROUP==0) LIST=initialSingle();
	if(GROUP==1) LIST=initialGroup();
	DIFF=P.FM.getDiff();
	CM=DataCM.f();
	DCM=DataCM.df(DIFF);
    }


    /**Get the initial lists of simplices*/
  public int[][] initialSingle() {
      int[] I=P.SM.N48;
      int[][] LIST={I};
      return(LIST);
  }

    public int[][] initialGroup() {
	int[][] LIST0=new int[48][4];

	int c=0;
	for(int i=0;i<4;++i) {
	    for(int j=0;j<4;++j) {
		for(int k=0;k<2;++k) {
		    for(int l=0;l<2;++l) {
			if(i!=j) {
			    int t=P.SM.S48[i][j][k][l].state;
			    if(t==1) {
				int[] I={i,j,k,l};
				LIST0[c]=I;
				++c;
			    }
			}
		    }
		}
	    }
	}
	int[][] LIST=new int[c][4];
	for(int i=0;i<c;++i) LIST[i]=LIST0[c-i-1];
	TOTAL=c;
	return(LIST);
    }


    /**main loop*/
    public void process() {
	int n=LIST.length;
        int k=random(n);
	int[] S=LIST[k];
	boolean test=findNegative(S);
	if(test==true) {
	    LIST=omit(LIST,k);
	    P.repaint();
	}
    }


    public boolean findNegative(int[] S) {
	int[][] a=ModuliSpace.simplex48(S[0],S[1],S[2],S[3]);
         double[] u=new double[6];
	 for(int q=0;q<6;++q) u[q]=Math.random();
	 double sum=0;
	 for(int q=0;q<6;++q) sum=sum+u[q];
	 for(int q=0;q<6;++q) u[q]=u[q]/sum;
	 double[] t=ModuliSpace2.map(a,u);
	boolean test=false;
	double f=CayleyMenger.f(t);
	double g=CayleyMenger.df(DIFF,t);
	if((f>0)&&(g<0)) {
	    test=antiCertify(S,u);
	}
	return(test);
    }

    /**Here is the one rigorous routine*/

    public boolean antiCertify(int[] S,double[] u) {
	BigInteger ZERO=new BigInteger("0");
	BigInteger[] v=nearbyBig(10,u);
	for(int i=0;i<6;++i) {
            if(v[i].compareTo(ZERO)<1) return(false);    //all terms positive
	}
	int[][] a=ModuliSpace.simplex48(S[0],S[1],S[2],S[3]);
	BigInteger[][] A=ModuliSpace.big(a);
	BigInteger[] tt=ModuliSpace.map(A,v);
	int sign1=CM.evaluateSign(tt);
	int sign2=DCM.evaluateSign(tt);
	if((sign1==1)&&(sign2==-1)) return(true);
	return(false);
    }

    /**This returns a big integer which, when divided by N,
       gives a number close to t.  The sum of the coordinates
       should be 24 N*/

    public BigInteger[] nearbyBig(int k,double[] t) {
	BigInteger[] u=new BigInteger[6];
	for(int i=0;i<6;++i) u[i]=nearby(k,t[i]);
	return(u);
    }

    /**This routine scales up by 10^k and takes the floor.
       Dividing back down by 10^k we get a controlled
       rational approximation to our original double.*/

    public static BigInteger nearby(int k,double t) {
	BigDecimal T=new BigDecimal(t);
	T=T.scaleByPowerOfTen(k);
	return(T.toBigInteger());
    }

    public BigInteger floor(BigDecimal D) {
	String S=D.toString();
	int n=S.indexOf(".");
	S=S.substring(0,n);
	BigInteger E=new BigInteger(S);
	return(E);
    }




    public int[][] omit(int[][] LIST,int q) {
	int n=LIST.length;
	int[][] LIST2=new int[n-1][4];
	int c=0;
	for(int i=0;i<n;++i) {
	    if(i!=q) {
		for(int j=0;j<4;++j) LIST2[c][j]=LIST[i][j];
		++c;
	    }
	}
	int i=LIST[q][0];
	int j=LIST[q][1];
	int k=LIST[q][2];
	int l=LIST[q][3];
	P.SM.S48[i][j][k][l].state=6;
	return(LIST2);
    }

    public int[][] initialList() {
	int[][] LIST=new int[48][8];
	int c=0;
	for(int i=0;i<4;++i) {
	    for(int j=0;j<4;++j) {
		for(int k=0;k<2;++k) {
		    for(int l=0;l<2;++l) {
			if(i!=j) {
			    int[] I={i,j,k,l};
			  P.SM.S48[i][j][k][l].state=1;
			  LIST[c]=I;
			  ++c;
			}
		    }
		}
	    }
	}
	return(LIST);
    }

    public int random(int n) {
	double x=Math.random();
	int m=(int)(Math.floor(n*x));
	return(m);
    }

}


