import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.math.*;


public class PositiveDominance implements Runnable {
    boolean HALT;
    PictureCanvas P;
    double TIME1,TIME2,VOLUME,TOTAL;
    Poly5[] LIST;
    int STEPS;
    int[][] MARKER=new int[48][4];
    int CURRENT;
    int PARTITION;
    int GROUP;
    boolean FAIL,NEGATIVE;

    /**Constructors for the class*/

    public PositiveDominance() {
	HALT=true;
    }

    public PositiveDominance(PictureCanvas PP) {
	HALT=true;
	this.P=PP;
    }

    /**Active thread*/

    public void run() {
	System.out.println("");  
	System.out.println("START");  
	preProcess();
	if(LIST.length==0) HALT=true;
	while(HALT==false)  process();
	postProcess();
	HALT=true; 
        P.repaint();
    }

    /**This routine sets the initial variables and also
       prevents the algorithm running on trivial data.*/

    public void preProcess() {	
        P.CM.clear();
	P.repaint();
        TIME1=System.currentTimeMillis();
	HALT=false;
	FAIL=false;
	NEGATIVE=false;
	VOLUME=0;
	STEPS=0;
	PARTITION=P.SM.CHOICE.mode;
	GROUP=P.GROUP.mode;
	CURRENT=0;	
        LIST=initialList();
	avoidForbidden();
    }

    /**This avoids running the program on empty data or in
       configurations that are not allowed*/

    public void avoidForbidden() {
	if((P.FM.LIN[0].val==0)&&(P.FM.LIN[1].val==0)) {
	    HALT=true;
	    System.out.println("trivial directional derivative");
	}

	if((PARTITION<2)&&(GROUP==1)) {
	    HALT=true;
	    System.out.println("group mode only works for P12 and P48");
	}

	if(LIST.length==0) {
	    HALT=true;
	    System.out.println("no functions are being tested.");
	}

    }



    /**This routine does the body of the algorithm*/

    public void process() { 
       if(HALT==true) return;
       ++STEPS;
       int n=LIST.length; 
       LIST=step(LIST);
       sendState(); 
       markCurrent();
       testFail();
       testPass();
       if(LIST.length==0) HALT=true;
       if(FAIL==true) HALT=true;
       P.repaint();
    }


    public void markCurrent() {
       int[] I=MARKER[CURRENT];
       if(PARTITION==2) P.SM.S12[I[0]][I[1]].state=5;
       if(PARTITION==3) P.SM.S48[I[0]][I[1]][I[2]][I[3]].state=5;
    }

    public void testFail() {
        if(GROUP==0) return;
        if(FAIL==true) { 
	   int s=3;
	   if(NEGATIVE==true) s=4;
           int[] I=MARKER[CURRENT];
	   LIST=purge(I); 
           if(PARTITION==2) P.SM.S12[I[0]][I[1]].state=s;
           if(PARTITION==3) P.SM.S48[I[0]][I[1]][I[2]][I[3]].state=s;
	   ++CURRENT;
	   VOLUME=Math.floor(VOLUME+1);
	   FAIL=false;
	   NEGATIVE=false;
       }
    }

    public void testPass() {
       if(GROUP==0) return;
       if(PARTITION<2) return;
       if(CURRENT==48) return;
       int[] I=MARKER[CURRENT]; 
       boolean test=scan(I);

       if((PARTITION==2)&&(test==false)) { 
	   P.SM.S12[I[0]][I[1]].state=2;
	   P.SM.STEPS12[I[0]][I[1]]=STEPS;
	   STEPS=0;
	   ++CURRENT;
       }  

       if((PARTITION==3)&&(test==false)) { 
	   P.SM.S48[I[0]][I[1]][I[2]][I[3]].state=2;
	   P.SM.STEPS48[I[0]][I[1]][I[2]][I[3]]=STEPS;
	   STEPS=0;
	   ++CURRENT;
       }  
    }


    /**This routine wraps up the algorithm with a printout.*/
    public void postProcess() {
	sendState();
	if(FAIL==false) {
	    if(VOLUME>TOTAL-.00000001) {
                System.out.println("PROGRESS "+" "+TOTAL+" "+VOLUME);
                System.out.println("STEPS "+" "+STEPS);
		int[] t=convertTime(TIME2-TIME1);
		System.out.println("TIME "+t[0]+"."+t[1]);
		System.out.println("DONE");
		System.out.println("");
	    }
	  else System.out.println("ABORTED");
	}
	else System.out.println("FAIL");
    }

    /**These routines load up the initial polynomials*/
    public Poly5[] initialList() {
	if((GROUP==0)||(PARTITION<2))  return(initialList0());
	if((GROUP==1)&&(PARTITION==2)) return(initialList1());
	if((GROUP==1)&&(PARTITION==3)) return(initialList2());
	return(null);
    }

    /**single mode*/
    public Poly5[] initialList0() {
	Poly6 P6=P.FM.getFunction();
	int[][] b=P.SM.getSimplex();
	Poly5 P5=P6.toCube(b);
	Poly5[] LIST={P5};
	if(P.SAVE.mode==1) saveToFile(b,P6,P5);
	TOTAL=1;
	return(LIST);
    }

    /**group mode for the 12 simplex mode*/
    public Poly5[] initialList1() {
	Poly5[] LIST0=new Poly5[12];
	Poly6 P6=P.FM.getFunction();

	int c=0;
	for(int i=0;i<4;++i) {
	    for(int j=0;j<4;++j) {
		if(i!=j) {
		   int t=P.SM.S12[i][j].state;
		   if(t==1) {
                      int[][] b=ModuliSpace.simplex12(i,j);
		      P.SM.STEPS12[i][j]=0;
		      LIST0[c]=P6.toCube(b);
		      int[] I={i,j,9,9};
		      LIST0[c].marker=I;
		      MARKER[c]=I;
		      int cc=c+1;
		      System.out.print("polys computed "+cc+"    ");
		      Debugger.printout(LIST0[c].marker);  
                      ++c;
		   }
		}
	    }
	}
	Poly5[] LIST=new Poly5[c];
	for(int i=0;i<c;++i) {
             LIST[i]=LIST0[c-i-1];
	}
	TOTAL=c;
	return(LIST);
    }


    /**group mode for the 48 simplex mode*/
    public Poly5[] initialList2() {
	Poly5[] LIST0=new Poly5[48];
	Poly6 P6=P.FM.getFunction();

	int c=0;
	for(int i=0;i<4;++i) {
	    for(int j=0;j<4;++j) {
		for(int k=0;k<2;++k) {
		    for(int l=0;l<2;++l) {
			if(i!=j) {
			    int t=P.SM.S48[i][j][k][l].state;
			    if(t==1) {
                               int[][] b=ModuliSpace.simplex48(i,j,k,l);
			       P.SM.STEPS48[i][j][k][l]=0;
			       LIST0[c]=P6.toCube(b);
			       int[] I={i,j,k,l};
			       LIST0[c].marker=I;
			       MARKER[c]=I;
			       int cc=c+1;
			       System.out.print("polys computed "+cc+"    ");
			       Debugger.printout(LIST0[c].marker);  
                               ++c;
			    }
			}
		    }
		}
	    }
	}
	Poly5[] LIST=new Poly5[c];
	for(int i=0;i<c;++i) LIST[i]=LIST0[c-i-1];
	TOTAL=c;
	return(LIST);
    }





    /**The main step of the algorithm*/
    public Poly5[] step(Poly5[] LIST0) {
	int last=LIST0.length-1;
	Poly5 Q=new Poly5(LIST0[last]);	
	if(Q.isNegativeAtOrigin()==true) {
             NEGATIVE=true;
	     FAIL=true;
	}
	P.repaint();


	boolean test=Q.isPositiveDominant();
	if(test==true) {
	    Poly5[] LIST1=new Poly5[last];
	    for(int i=0;i<last;++i) LIST1[i]=new Poly5(LIST0[i]);
	    VOLUME=VOLUME+volume(Q);
	    return(LIST1);
	}

	Poly5[] LIST1=new Poly5[last+2];
	for(int i=0;i<last;++i) LIST1[i]=new Poly5(LIST0[i]);
	LIST1[last+0]=Q.subdivide(0);
	LIST1[last+1]=Q.subdivide(1);
	return(LIST1);
    }

    /**This routine scans the list to see if there are any
       markers of the given kind*/

    public boolean scan(int[] K) {
	for(int i=0;i<LIST.length;++i) {
	    if(match(LIST[i].marker,K)==true) return(true);
	}
	return(false);
    }

    /**This routine removes from the list all polynomials which
       have the given marker.  It is used when the algorithm fails
       for some polynomial in the list (in group mode), so that the
       algorithm can continue working on the next initial polynomial
       after it fails.*/

    public Poly5[] purge(int[] K) {
	Poly5[] LIST0=new Poly5[LIST.length];
	int c=0;
	for(int i=0;i<LIST.length;++i) {
	    if(match(LIST[i].marker,K)==false){
		LIST0[c]=new Poly5(LIST[i]);
		++c;
	    }
	}
	Poly5[] LIST1=new Poly5[c];
	for(int i=0;i<c;++i) LIST1[i]=new Poly5(LIST0[i]);
	return(LIST1);
    }


    public static boolean match(int[] a,int[] b) {
	if(a.length!=b.length) return(false);
	for(int i=0;i<a.length;++i) {
	    if(a[i]!=b[i]) return(false);
	}
	return(true);
    }


    /**These routines communicate with the calculation monitor*/
    public void sendState() { 
       int n=LIST.length;
         TIME2=System.currentTimeMillis();
         P.CM.TIME=TIME2-TIME1;
         if(n>0) {
             P.CM.AGE=LIST[n-1].age;
	     if(LIST[n-1].age[4]==P.TOLERANCE.val) {
                 FAIL=true;
	     }
	 }
         P.CM.VOLUME=VOLUME;
         P.CM.TOTAL=TOTAL;
         P.CM.STEPS=STEPS;
    }

    /**converts the time from milliseconds to minutes and seconds*/
    public static int[] convertTime(double t) {
	double u1=t/1000;  //seconds
	double u2=Math.floor(u1/60); //minutes
	double u3=u1-60*u2;  
	int a=(int)(u2);
	int b=(int)(u3);
	int[] A={a,b};
	return(A);
    }



    /**Computes the volume of a pair (G,I). Even though all
       polynomials have [0,1]^5 as their domain, there is an
       implicit understanding that a given polynomial on the
       list takes on the same values on a sub-rectangle R that
       the initial polynomial takes on [0,1]^5/  What is computed
       is the volume of R.  One can glean this information from
       the age vector I.*/

    public double volume(Poly5 Q) {
	double x=1;
	for(int i=0;i<5;++i) x=x*Math.pow(.5,Q.age[i]);
	return(x);
    }


    /**This routine write the initial data to a file. It is only
       called in the single mode.*/

    public void saveToFile(int[][] b,Poly6 P6,Poly5 P5) {
	Output OUT=new Output("save_polys");
	OUT.println("(*Mathematica Usable File*)");
	OUT.println("");
	OUT.println("(*The simplex*)");
	OUT.print("simplex={");
	for(int i=0;i<6;++i) {
	    OUT.print("{");
	    for(int j=0;j<6;++j) {
		OUT.print(b[i][j]);
		if(j<5) OUT.print(",");
		else OUT.print("}");
	    }
	    if(i<5) OUT.print(",");
	    else OUT.print("};");
	}
	OUT.println("");
	OUT.println("");

	OUT.println("(*the directional derivative*)");
	int[] q=P.FM.getDiff();
	OUT.print("diff={");
	for(int i=0;i<6;++i) {
	    OUT.print(q[i]);
	    if(i<5) OUT.print(",");
	    else OUT.print("};");
	}
	OUT.println("");
	OUT.println("");

	OUT.println("(*the linear combination coeff1 f + coeff2 df*)");
	OUT.print("coeff={");
	OUT.print(P.FM.LIN[0].val);
	OUT.print(",");
	OUT.print(P.FM.LIN[1].val);
	OUT.print("};");
	OUT.println("");
	OUT.println("");

	OUT.println("(*initial polynomial*)");
	P6.print(OUT,"P");
	OUT.println("");
	OUT.println("(*cube-transformed polynomial*)");
	P5.print(OUT,"Q");
	OUT.println("");
	OUT.println("(*end of file*)");
    }



}